------------------------------------------------------------------------------
-- SellableItemDrops - Logging your sellable item drops
------------------------------------------------------------------------------
-- Modules/GUI.lua - Interface
--
-- Author: Expelliarm5s / June 2019 / All Rights Reserved
--
-- Version 0.4.11
------------------------------------------------------------------------------
local addonName, addon = ...
local GUI = addon:NewModule("GUI", "AceEvent-3.0", "AceHook-3.0", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local AceGUI = LibStub("AceGUI-3.0")
local private = {}
------------------------------------------------------------------------------

GUI.frScaleValue = 0.8
GUI.view = nil
GUI.isShrinked = false
GUI.lastWidth = 0
GUI.rarityList = {}
for i = 0, 8 do
	GUI.rarityList[i] = ITEM_QUALITY_COLORS[i].hex ..
		_G[format("ITEM_QUALITY%d_DESC", i)] ..
		FONT_COLOR_CODE_CLOSE
end

GUI.timeFormat = {}
GUI.timeFormat["ago"]  = L["_ Hr. _ Min. ago"]
GUI.timeFormat["%d.%m.%Y %H:%M:%S"] = "DD.MM.YYYY HH:MM"
GUI.timeFormat["%Y-%m-%d %H:%M:%S"] = "YYYY-MM-DD HH:MM"

GUI.guiTabs = {}
GUI.guiTabs.texts = {}
GUI.guiTabs.order = {}
GUI.guiTabsSelect = {}

GUI.TITLEWIDTH_ITEM = 0.31
GUI.TITLEWIDTH_VALUE = 0.17
GUI.TITLEWIDTH_QTY = 0.04
GUI.TITLEWIDTH_PLAYER = 0.10
GUI.TITLEWIDTH_ZONE = 0.20
GUI.TITLEWIDTH_TIME = 0.18
GUI.TITLEWIDTH_GAP = 0.01

GUI.adapt = GUI.adapt or {}

-- printing debug info
function GUI:DebugPrintf(...)
	if addon.isDebug then
		GUI:Printf(...)
	end
end

-- print info on loaded session data
function GUI:ShowInfo()
	if ((addon.db.realm.sesTime + 3 >= time()) and addon.db.realm.sesEnd == 0) then
		addon:Printf(L["Starting a new session."])
	elseif (addon.db.realm.sesEnd == 0) then
		addon:Printf(L["Continuing a previous session. Current Looted Item Value is %s"], GetCoinTextureString(addon.db.realm.sesMV))
	else
		addon:Printf(L["Session is stopped. A new session starts after saving or wiping the current session data."])
	end
	addon:Printf(L["  Quality filter is >=%s."], GUI.rarityList[addon.db.global.minRarity])
	addon:Printf(L["  Noteworthy items are worth >%s."], addon.db.global.minNoteworthy)
	mvSourcesList = addon.Item:GetPriceSources() or {}
	addon:Printf(L["  Price source is '%s'."],
		mvSourcesList[addon.db.global.mvSource] or addon.db.global.mvSource)
	addon:Print(L["Good luck farming items!"])
end

-- set main windows status line
function GUI:SetStatusLine(txt)
	if GUI.display then
		GUI.display:SetStatusText(txt)
	end
end

-- print info on ItemStrings
function GUI:PrintItemStrings(link)
	if not link then
		return
	end

	local q = addon.Item:GetItemQuality(link)
	if not q then
		q = 6
	end

	local l = ""
	if type(link) == "string" and link:match("^\124(.+)\124h") then
		l = gsub(link, "\124", "/")
		l = "L=" .. ITEM_QUALITY_COLORS[q].hex .. l .. FONT_COLOR_CODE_CLOSE .. " "
	end

	GUI:Printf("%sI=%s%s%s IB=%s%s%s ID=%s%s%s",
			l,
			ITEM_QUALITY_COLORS[q].hex,
			addon.Item:GetItemString(link) or "nil",
			FONT_COLOR_CODE_CLOSE,
			ITEM_QUALITY_COLORS[q].hex,
			addon.Item:GetBaseItemString(link) or "nil",
			FONT_COLOR_CODE_CLOSE,
			ITEM_QUALITY_COLORS[q].hex,
			tostring(addon.Item:GetItemID(link) or "nil"),
			FONT_COLOR_CODE_CLOSE
			)
end

-- return formatted time
function GUI:TimeToString(t)
	local res
	if addon.db.global.timeFormat == "ago" then
		local lTime = time() - t
		if (lTime == 0) then
			res = L["now"]
		else
			if GUI.display and GUI.adapt.width and GUI.adapt.width <= 600 then
				if (floor(lTime / 60) * 60) == 0 then
					res = L["now"]
				else
					res = format(L["%s ago"],  SecondsToTime(floor(lTime / 60) * 60) or "?")
				end
			else
				res = format(L["%s ago"],  SecondsToTime(lTime) or "?")
			end
		end
	elseif (addon.db.global.timeFormat) then
		if GUI.display and GUI.adapt.width and GUI.adapt.width <= 600 then
			if addon.db.global.timeFormat == "%Y-%m-%d %H:%M:%S" then
				res = date("%H:%M:%S", t)
			end
			if addon.db.global.timeFormat == "%d.%m.%Y %H:%M:%S" then
				res = date("%H:%M:%S", t)
			end
		else
			res = date(addon.db.global.timeFormat or addon.timeFormat, t)
		end
	end
	res = GUI.ShrinkedTime(res)
	return res
end

-- parse money string to copper
function GUI:ToCopper(str)
	local c = tonumber(strmatch(str, "([0-9]+)c"))
	local s = tonumber(strmatch(str, "([0-9]+)s"))
	local g = tonumber(strmatch(str, "([0-9]+)g"))
	if not c and not s and not g then return end

	local copper = (c or 0)
	copper = copper + (s or 0) * COPPER_PER_SILVER
	copper = copper + (g or 0) * COPPER_PER_GOLD

	return copper
end

-- build money string from copper
function GUI:FromCopper(copper, isVS)
	if not tonumber(copper) then return end

	local g = floor(copper / COPPER_PER_GOLD)
	local s = floor((copper - (g * COPPER_PER_GOLD)) /  COPPER_PER_SILVER)
	local c = copper - (g * COPPER_PER_GOLD) - (s * COPPER_PER_SILVER)

	local result = ""
	
	local vsC = ""
	local vsR = ""
	if isVS then
		vsC = ITEM_QUALITY_COLORS[7].hex	
		vsR = FONT_COLOR_CODE_CLOSE
	end
	
	if not addon.db.global.discardSilver then
		if not addon.db.global.discardCopper then
			if (s > 0) or (g > 0) then
				result = vsC .. format("%02d", c) .. vsR .. "|cffeda55fc|r"
			else
				result = vsC .. format("%d", c) .. vsR .. "|cffeda55fc|r"
			end
		end
		if g > 0 then
			result = vsC .. format("%02d", s) .. vsR .. "|cffc7c7cfs|r " .. result
		else
			if s > 0 then
				result = vsC .. format("%d", s) .. vsR .. "|cffc7c7cfs|r " .. result
			end
		end
	end
	if g > 0 then
		local gs = tostring(g)
		gs = gsub(gs, "(%d)(%d%d%d)$", "%1" .. LARGE_NUMBER_SEPERATOR .. "%2")
		result = vsC .. gs .. vsR .. "|cffffd70ag|r " .. result
	end

	return result
end

------------------------------------------------------------------------------
-- Tabs

-- call selected tab
GUI.guiTabsOnSelected = function(container, event, group)
	GUI:DebugPrintf("guiTabsOnSelected: %s", group)
	if not container then
		GUI:DebugPrintf("guiTabsOnSelected: container is nil")
	end

	container:ReleaseChildren()
	GUI.guiTabs.lastSelected = group
	if GUI.guiTabsSelect[group] then
		GUI.view = group
		GUI.container = container
		GUI.guiTabsSelect[group](container, group)
	end
end

-- do things on logout
function GUI:OnLogout()
	if GUI.adapt then
		if GUI.adapt.shPlayer ~= nil then
			addon.db.global.shPlayer = GUI.adapt.shPlayer
		end
		if GUI.adapt.shZone ~= nil then
			addon.db.global.shZone = GUI.adapt.shZone
		end
	end
end

-- load GUI with last selected tab
function GUI:Load(optSelectTab)
	GUI:DebugPrintf("Load()")

	-- prepare blacklisted and vendorsell items
	addon.Data:IsBlacklisted("")
	addon.Data:IsVendorsell("")

	-- check if the currently used price source is available and check for sanity of options
	addon:CheckSettings()

	-- create main window
	if not GUI.display then
		GUI:DebugPrintf("Load(): create new frame")

		local display = AceGUI:Create("SIDFrame")
		addon.db.global.ui = addon.db.global.ui or {}
		display:SetStatusTable(addon.db.global.ui)

		local title = "|cFF33FF99" .. addon.METADATA.NAME .. " (" .. addon.METADATA.VERSION .. ")|r"
		display:SetTitle(title)
		display:SetLayout("Fill")
		display:SetStatusText("--")
		display:EnableResize(true)
		display:SetCallback("OnClose", function(_display)
			GUI:DebugPrintf("OnClose(%s)", tostring(_display.frame))
			AceGUI:Release(_display)
			GUI.display = nil
			-- collectgarbage()
		end)

		GUI.adapt.width = display.frame:GetWidth()
		-- GUI:Printf("GetWidth()=%s", GUI.adapt.width)
		if display and display.OnWidthSet then
			local ows = display.OnWidthSet
			display.OnWidthSet = function(_self, _width)
				ows(_self, _width)
				-- GUI:Printf("OnWidthSet=%s", _width)
				if private and GUI.adapt then
					GUI.adapt.width = _width
				end
			end
		end

		GUI.display = display
	end

	if #GUI.display.children > 0 then
		GUI.display:ReleaseChildren()
	end

	local fmtstr = #GUI.guiTabs.order > 6 and " %s s" or "   %s   "
	for i, name in ipairs(GUI.guiTabs.order) do
		GUI.guiTabs[i] = {
			value = name,
			text = fmtstr:format(GUI.guiTabs.texts[name].title),
			disabled = GUI.guiTabs.texts[name].disabled,
		}
	end

	local tabs = AceGUI:Create("SIDTabGroup")
	tabs:SetLayout("Flow")
	tabs:SetTabs(GUI.guiTabs)
	tabs:SetCallback("OnGroupSelected", GUI.guiTabsOnSelected)
	tabs:SetCallback("OnTabEnter", function(_tabs,event,value,tab)
		GUI:SetStatusLine(GUI.guiTabs.texts[value].desc)
	end)
	tabs:SetCallback("OnTabLeave", function() GUI:SetStatusLine("") end)

	GUI.display:AddChildren(tabs)

	if optSelectTab and optSelectTab ~= "" then
		tabs:SelectTab(optSelectTab)
	else
		GUI.guiTabs.lastSelected = GUI.guiTabs.lastSelected or GUI.guiTabs.order[1]
		tabs:SelectTab(GUI.guiTabs.lastSelected)
	end

	GUI.display:Show()

	-- GUI:Printf("GUI.isShrinked=%s", tostring(GUI.isShrinked))
	if GUI.isShrinked then
		if GUI.view == "session" then
			GUI.isShrinked = false
			GUI:Shrink(true)
			GUI.lastWidth = 0
		end
	else
		GUI.isShrinked = false
	end

	return GUI.display
end

function GUI:Shrink(shrinkMe)
	-- GUI:Printf("Shrink(%s)", tostring(shrinkMe))
	if shrinkMe and not GUI.isShrinked then
		-- GUI:Printf("  shrink it!")
		-- FIXME: release children
		GUI.isShrinked = true
		GUI.container:SelectTab("session")
		for i=1, #GUI.container.tablist do
			GUI.container.tablist[i] = nil
		end
		GUI.container:BuildTabs()
		GUI.container.border:SetPoint("TOPLEFT", 1, - 8)
		GUI.lastWidth = GUI.container.frame:GetWidth()
		return
	end
	if not shrinkMe and GUI.isShrinked then
		-- GUI:Printf("  unshrink it!")
		GUI.isShrinked = false
		GUI:Load()
		return
	end
end

------------------------------------------------------------------------------
-- Private Functions

function GUI.DataFrame_OnEnter(btn, data)
	if btn.btnCnt then
		GUI:SetStatusLine(GUI.container.titleStatusText[btn.btnCnt])
	end

	if type(data) == "table" and data.link and type(data.link) == "string" then
		-- GUI:DebugPrintf("Tooltip for %s", data.link)
		if data.link:match("battlepet") then
			local _, speciesID, level, breedQuality, maxHealth, power, speed, _ = strsplit(":", data.link)
			-- FrameXML/BattlePetTooltip.lua
			local a, b = pcall(BattlePetToolTip_Show,
				tonumber(speciesID), tonumber(level), tonumber(breedQuality), tonumber(maxHealth), tonumber(power), tonumber(speed))
			if not a then
				GUI:Printf(b)
			end
		else
			GameTooltip:SetOwner(btn, "ANCHOR_TOPLEFT")
			local a, b = pcall(GameTooltip.SetHyperlink, GameTooltip, data.link)
			if not a then
				GUI:Printf(b .. " " .. tostring(data.link))
			end
		end
		
		-- StatusLine
		GUI:SetStatusLine(GUI.container.statusText)
	end
end

function GUI.DataFrame_OnLeave(btn, data)
	if btn.btnCnt then
		GUI:SetStatusLine()
	end
	if type(data) == "table" then
		GameTooltip:Hide()
		BattlePetTooltip:Hide()
		GUI:SetStatusLine()
	end
end

-- prepare item data
function GUI.GetItemData(items, lfilters, showBOP)
	GUI:DebugPrintf("GetItemData()")
	local content = {}
	local filters = lfilters or {}

	for link, data in pairs(items) do
		if #data.drops > 0 and not GUI.IsItemFiltered(link, data, filters) then
			for _, record in ipairs(data.drops) do
				if not GUI.IsRecordFiltered(record, filters) then
					local isVariant = data.iString ~= data.iBaseString
					local lItem = ""
					local marketValue = 0
					local vs = false
					if (addon.db.global.useNotTheRealItemString) then
						lItem = addon.Item:GetItemLink(data.iBaseString)
						if not lItem then
							lItem = link
							-- GUI:DebugPrintf("GetItemData: GetLink for %s is nil", data.iBaseString)
						end
						if (isVariant) then
							lItem = lItem .. ITEM_QUALITY_COLORS[5].hex .. " !" .. FONT_COLOR_CODE_CLOSE
						end
						marketValue, vs = addon.Data:GetItemValue(link)
					else
						lItem = link
						marketValue, vs = addon.Data:GetItemValue(link)
					end

					local name = addon.Item:GetItemName(lItem, link)
					if not name then
						name = L["Unknown"]
						GUI:DebugPrintf("GetItemData: GetName for %s is nil",link)
					end

					if (addon.Item:GetItemIsBOP(link)) then
						lItem = lItem .. ITEM_QUALITY_COLORS[8].hex .. " BOP" .. FONT_COLOR_CODE_CLOSE
					end

					if (CanIMogIt) then
						if not (addon.isInfight) then
							if (CanIMogIt:PlayerKnowsTransmog(link)) then
								-- lItem = lItem .. ITEM_QUALITY_COLORS[2].hex .. " TM" .. FONT_COLOR_CODE_CLOSE
							else
								if (CanIMogIt:CharacterCanLearnTransmog(link)) then
									lItem = lItem .. ITEM_QUALITY_COLORS[5].hex .. " TM" .. FONT_COLOR_CODE_CLOSE
								end
							end
						end
					end

					local data = {}
					data.link = link
					data.time = record.time
					data.bagid = record.bagid or 0
					data.slotid = record.slotid or 0
					data.data = {}
					data.data[1] = {lItem, name}

					local fc = (GUI:FromCopper(marketValue, vs) or "|cff999999---|r") .. "  "
					-- GUI:Printf("%s: value=%s, string=%s", link, tostring(marketValue), fc)
	
					data.data[2] = {fc, marketValue or 0}
					data.data[3] = {record.quantity .. " ", record.quantity}
					if (not addon.db.global.shPlayer) then
						data.data[4] = nil
					else
						data.data[4] = {record.player, record.player}
					end
					if (GUI.view == "inventory") or (GUI.view == "bank") then
						data.data[5] = {record.zone, record.zone}
					else
						if (not addon.db.global.shZone) then
							data.data[5] = nil
						else
							data.data[5] = {record.zone, record.zone}
						end
					end
					data.data[6] = {GUI:TimeToString(record.time), record.time}
					if (GUI.view == "session") and (not addon.db.global.shTime) then
						data.data[6] = nil
					else
						if (GUI.view == "inventory") then
							data.data[6] = nil
						end
						if (GUI.view == "bank") then
							data.data[6] = nil
						end
					end
					tinsert(content, data)
				end
			end
		end
	end
	return content
end

-- create and return button/other widget
function GUI.AddButton(widgetType, label, status)
	-- GUI:DebugPrintf("AddButton %s, %s", widgetType, label)
	if not status then
		widgetType, label, status = "Button", widgetType, label
	end
	local button = assert(AceGUI:Create(widgetType))
	if button.SetText then
		button:SetText(tostring(label))
	end
	button:SetCallback("OnEnter", function() GUI:SetStatusLine(status) end)
	button:SetCallback("OnLeave", function() GUI:SetStatusLine("") end)
	return button
end


-- prepate list of players from item data
function GUI.playerListFromItems(items)
	local playerList = {}
	if items then
		for _, data in pairs(items) do
			if data.drops then
				for _, record in ipairs(data.drops) do
					playerList[record.player] = record.player
				end
			end
		end
	end
	playerList["all"] = L["All"]
	sort(playerList, function(a, b) return (a.player or 0) < (b.player or 0) end)
	return playerList
end

-- prepate list of zones from item data
function GUI.zoneListFromItems(items)
	local zoneList = {}
	if items then
		for _, data in pairs(items) do
			if data.drops then
				for _, record in ipairs(data.drops) do
					zoneList[record.zone] = record.zone
				end
			end
		end
	end
	zoneList["all"] = L["All"]
	sort(zoneList, function(a, b) return (a.zone or 0) < (b.zone or 0) end)
	return zoneList
end

-- check if item is filtered
function GUI.IsItemFiltered(link, data, filters)
	local lItemString = link

	if (addon.db.global.useNotTheRealItemString) then
		lItemString = addon.Item:GetBaseItemString(lItemString)
	end

	local rarity = addon.Item:GetItemQuality(lItemString, link)
	if (rarity) then
		if filters.rarity and rarity < filters.rarity then
			return true
		end
	else
		-- GUI:DebugPrintf("Rarity for %s/%s is nil.", lItemString, link)
	end

	local name = addon.Item:GetItemName(lItemString, link)
	if not name then
		return false
	end

	if filters.name and not strfind(strlower(name), strlower(filters.name)) then
		return true
	end

end

-- check if record is filtered
function GUI.IsRecordFiltered(record, filters)
	local SECONDS_PER_DAY = 24 * 60 * 60

	if filters.player and record.player ~= filters.player then
		return true
	end

	if filters.zone and record.zone ~= filters.zone then
		return true
	end

	if filters.time and floor(record.time/SECONDS_PER_DAY) < (floor(time()/SECONDS_PER_DAY) - filters.time) then
		return true
	end
end

function GUI.ShrinkedTime(t)
	local res = t
	if GUI.isShrinked and res then
		res = gsub(res, "Std.", "::")
		res = gsub(res, "Min.", "Min")
		res = gsub(res, "vor", "v.")
	end
	return res
end

-- EOF

